from __future__ import annotations

import functools
import pathlib
import typing

from cryptography.fernet import Fernet

from .vault import key_from_file, VaultToPlainYAML


class ContextProvider:
    def __init__(
        self, work_dir: pathlib.Path, key_path: typing.Union[str, pathlib.Path]
    ):
        self.work_dir = work_dir
        self.key_path = key_path
        self._fernet: typing.Optional[Fernet] = None

    def fernet(self) -> Fernet:
        if self._fernet is None:
            self._fernet = Fernet(key_from_file(self.work_dir / self.key_path))
        return self._fernet

    def load_context(self, path: pathlib.Path) -> typing.Mapping[str, typing.Any]:
        with open(path) as yaml_file:
            # We don't pass the property directly as this would try to load the
            # key from file which won't work when no key file is provided.
            # This is a valid use case e.g. when the context doesn't use
            # encrypted values.
            return VaultToPlainYAML(functools.partial(self.fernet)).load(yaml_file)



import base64
import json
import typing

import jinja2


def to_json(value: typing.Any) -> str:
    return json.dumps(value)


def b64encode(value: typing.Union[str, bytes]) -> str:
    # Only do str to bytes conversion
    if isinstance(value, str):
        value = value.encode("utf8")
    encoded_value = base64.b64encode(value)
    return encoded_value.decode("utf8")


class JinjaEnvironment(jinja2.Environment):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.filters["b64encode"] = b64encode
        self.filters["to_json"] = to_json

    def eval_expression(
        self, src: str, context: typing.Mapping[str, typing.Any]
    ) -> str:
        expr = self.compile_expression(src)
        return expr(context)

    def eval_template(
        self, template: str, context: typing.Mapping[str, typing.Any]
    ) -> str:
        tmpl = self.from_string(template)
        return tmpl.render(**context)



import collections
import pathlib
import typing

from ruamel.yaml import YAML

from .context import ContextProvider
from .yaml import KonverterYAML

DEFAULT_PROVIDER = {
    "default": {"key_path": ".konverter-vault",},
}


class Konverter:
    def __init__(self, templates, context, work_dir):
        self.templates = templates
        self.context = context
        self.work_dir = work_dir

    def render(self, out_file):
        yaml = KonverterYAML(self)
        # We want to have the documents from different files
        # separated by "---".
        yaml.explicit_start = True  # type: ignore
        for template_path in self.templates:
            with open(template_path) as template:
                yaml.render(template, out_file)

    @classmethod
    def from_file(cls, config_path: typing.Union[str, pathlib.Path]):
        config_path = pathlib.Path(config_path)
        work_dir = config_path.parent.absolute()

        with open(config_path) as config_file:
            cfg = YAML().load(config_file)

        return cls.from_dict(cfg, work_dir)

    @classmethod
    def from_dict(cls, config, work_dir):
        templates = list(
            cls._collect_templates(
                (pathlib.Path(p) for p in config["templates"]), work_dir
            )
        )
        providers = dict(
            cls._create_providers(
                collections.ChainMap(config.get("providers", {}), DEFAULT_PROVIDER),
                work_dir,
            )
        )
        context = collections.ChainMap(
            *cls._load_context(config["context"], providers, work_dir)
        )
        return cls(templates, context, work_dir)

    @staticmethod
    def _collect_templates(
        templates: typing.Iterable[typing.Union[str, pathlib.Path]],
        work_dir: pathlib.Path,
    ) -> typing.Generator[pathlib.Path, None, None]:
        for template_path in templates:
            path = work_dir / template_path
            if path.is_file():
                yield path
            elif path.is_dir():
                yield from sorted(path.glob("**/*.y[a]ml"))
            else:
                raise RuntimeError(f"Template path '{path}' not found")

    @staticmethod
    def _create_providers(
        providers, work_dir: pathlib.Path,
    ) -> typing.Generator[typing.Tuple[str, ContextProvider], None, None]:
        for name, provider in providers.items():
            yield name, ContextProvider(
                key_path=pathlib.Path(provider["key_path"]).expanduser(),
                work_dir=work_dir,
            )

    @staticmethod
    def _load_context(context, providers, work_dir: pathlib.Path):
        for ctx in context:
            if isinstance(ctx, str):
                provider, context_path = "default", ctx
            else:
                provider, context_path = ctx["provider"], ctx["path"]
            path = work_dir / context_path
            if path.is_file():
                yield providers[provider].load_context(path)
            elif path.is_dir():
                for file_path in sorted(path.glob("**/*.y[a]ml")):
                    yield providers[provider].load_context(file_path)



from __future__ import annotations

import json
import pathlib
import shutil
import tempfile
import typing

import click
from cryptography.fernet import Fernet
from ruamel.yaml.nodes import ScalarNode

from .yaml import BaseYAML, KonvertType

if typing.TYPE_CHECKING:
    from ruamel.yaml.constructor import Constructor
    from ruamel.yaml.representer import Representer

DEFAULT_VAULT_KEY_PATH = ".konverter-vault"


class KonvertEncrypt(KonvertType):
    yaml_tag = "!k/encrypt"

    @classmethod
    def to_yaml(
        cls, representer: Representer, instance: KonvertType, yaml
    ) -> ScalarNode:
        value = yaml.fernet.encrypt(bytes(instance.node.value, encoding="utf8"))
        return representer.represent_scalar(
            tag=KonvertVault.yaml_tag,
            value=str(value, encoding="utf8"),
            style=instance.node.style,
        )


class KonvertVault(KonvertType):
    yaml_tag = "!k/vault"

    @classmethod
    def to_yaml(
        cls, representer: Representer, instance: KonvertType, yaml
    ) -> ScalarNode:
        value = yaml.fernet.decrypt(bytes(instance.node.value, encoding="utf8"))
        return representer.represent_scalar(
            tag=KonvertEncrypt.yaml_tag,
            value=str(value, encoding="utf8"),
            style=instance.node.style,
        )


class KonvertVaultValue(KonvertVault):
    @classmethod
    def from_yaml(cls, constructor: Constructor, node: ScalarNode, yaml) -> ScalarNode:
        value = yaml.fernet.decrypt(bytes(node.value, encoding="utf8"))
        return constructor.construct_scalar(
            ScalarNode(
                tag="tag:yaml.org,2002:str",
                value=str(value, encoding="utf8"),
                style=node.style,
            )
        )


class KonvertEncryptValue(KonvertEncrypt):
    @classmethod
    def from_yaml(cls, constructor: Constructor, node: ScalarNode, yaml) -> ScalarNode:
        return constructor.construct_scalar(
            ScalarNode(tag="tag:yaml.org,2002:str", value=node.value, style=node.style)
        )


class VaultYAML(BaseYAML):
    def __init__(self, fernet: typing.Callable[[], Fernet]):
        super().__init__()
        if isinstance(fernet, Fernet):
            self.fernet: Fernet = fernet
        else:
            self._lazy_fernet = fernet

    def __getattr__(self, name):
        if name == "fernet":
            self.fernet = self._lazy_fernet()
            return self.fernet
        raise AttributeError(name)


class VaultToEditableYAML(VaultYAML):
    def __init__(self, fernet: typing.Callable[[], Fernet]):
        super().__init__(fernet)
        self.register_type(KonvertVault)

    def convert(self, vault: typing.IO[str], decrypted: typing.IO[str]) -> None:
        data = self.load(vault)
        if data is None:
            return
        self.dump(data, stream=decrypted)


class EditableToVaultYAML(VaultYAML):
    def __init__(self, fernet: typing.Callable[[], Fernet]):
        super().__init__(fernet)
        self.register_type(KonvertEncrypt)

    def convert(self, decrypted: typing.IO[str], vault: typing.IO[str]) -> None:
        data = self.load(decrypted)
        if data is None:
            return
        self.dump(data, stream=vault)


class VaultToPlainYAML(VaultYAML):
    def __init__(self, fernet: typing.Callable[[], Fernet]):
        super().__init__(fernet)
        self.register_type(KonvertEncryptValue)
        self.register_type(KonvertVaultValue)

    def convert(self, encrypted: typing.IO[str], plain: typing.IO[str]) -> None:
        data = self.load(encrypted)
        if data is None:
            return
        self.dump(data, stream=plain)


def key_from_file(key_path: typing.Union[str, pathlib.Path]) -> bytes:
    with open(key_path, "rb") as kfile:
        return kfile.read()


@click.group()
@click.option(
    "--key-path", default=DEFAULT_VAULT_KEY_PATH, type=click.Path(dir_okay=False),
)
@click.pass_context
def cli(ctx: click.Context, key_path: str) -> None:
    ctx.ensure_object(dict)
    if ctx.invoked_subcommand == "keygen":
        ctx.obj["key_path"] = key_path
        return

    if not pathlib.Path(key_path).exists():
        raise click.BadParameter("Key file not found", param_hint="key_path")
    ctx.obj["fernet"] = Fernet(key_from_file(key_path))


@cli.command()
@click.pass_context
def keygen(ctx: click.Context) -> None:
    key_path = ctx.obj["key_path"]
    if pathlib.Path(key_path).exists():
        click.confirm(
            "Key exists! Overwrite with new key?", default=False, err=True, abort=True,
        )
    with open(key_path, "wb") as key_file:
        key_file.write(Fernet.generate_key())


@cli.command()
@click.argument("file_path", type=click.Path(exists=True, dir_okay=False))
@click.pass_context
def encrypt(ctx: click.Context, file_path: str):
    fernet = ctx.obj["fernet"]

    encrypt_yaml = EditableToVaultYAML(fernet)
    with tempfile.NamedTemporaryFile(mode="wt", suffix=".yaml") as tmp_file:
        with open(file_path, "r") as yaml_file:
            encrypt_yaml.convert(yaml_file, tmp_file)

        tmp_file.flush()
        shutil.copy(tmp_file.name, file_path)


@cli.command()
@click.argument("file_path", type=click.Path(exists=True, dir_okay=False))
@click.pass_context
def decrypt(ctx: click.Context, file_path: str):
    fernet = ctx.obj["fernet"]

    decrypt_yaml = VaultToEditableYAML(fernet)
    with tempfile.NamedTemporaryFile(mode="wt", suffix=".yaml") as tmp_file:
        with open(file_path, "r") as yaml_file:
            decrypt_yaml.convert(yaml_file, tmp_file)

        tmp_file.flush()
        shutil.copy(tmp_file.name, file_path)


@cli.command()
@click.argument("file_path", type=click.Path(exists=True, dir_okay=False))
@click.pass_context
def edit(ctx: click.Context, file_path: str):
    fernet = ctx.obj["fernet"]

    decrypt_yaml = VaultToEditableYAML(fernet)
    with tempfile.NamedTemporaryFile(mode="w+t", suffix=".yaml") as tmp_file:
        with open(file_path, "r+") as yaml_file:
            decrypt_yaml.convert(yaml_file, tmp_file)
        tmp_file.flush()

        while True:
            click.edit(filename=tmp_file.name)
            tmp_file.flush()
            tmp_file.seek(0)
            try:
                encrypt_yaml = EditableToVaultYAML(fernet)
                with tempfile.NamedTemporaryFile(mode="wt") as out_tmp:
                    encrypt_yaml.convert(tmp_file, out_tmp)
                    out_tmp.flush()
                    shutil.copy(out_tmp.name, file_path)
                break
            except Exception:
                click.echo("Failed to load and encrypt file!", err=True)
                click.confirm(
                    "Continue editing? (if not, changes will be lost)",
                    default=True,
                    err=True,
                    abort=True,
                )


def to_terraform_format(data: dict) -> dict:
    def _convert(v):
        if isinstance(v, str):
            return v
        if isinstance(v, (float, int, bool)):
            return str(v)
        raise TypeError("Terraform JSON output only supports string values")

    return {k: _convert(v) for k, v in data.items() if v is not None}


@cli.command()
@click.argument("file_path", type=click.Path(exists=True, dir_okay=False))
@click.option(
    "-f",
    "--format",
    "output_format",
    type=click.Choice(["yaml", "json", "terraform"], case_sensitive=False),
    default="yaml",
)
@click.pass_context
def show(ctx: click.Context, file_path: str, output_format: str):
    fernet = ctx.obj["fernet"]

    encrypt_yaml = VaultToPlainYAML(fernet)
    with open(file_path, "r+") as yaml_file:
        with click.open_file("-", "wt") as stdout:
            data = encrypt_yaml.load(yaml_file)
            if output_format == "json":
                json.dump(data, stdout, indent=2)
            elif output_format == "terraform":
                json.dump(to_terraform_format(data), stdout, indent=2)
            else:
                encrypt_yaml.dump(data, stdout)


if __name__ == "__main__":
    cli()



from __future__ import annotations

import abc
import copy
import functools
import typing

import jinja2
from ruamel.yaml import YAML
from ruamel.yaml.constructor import RoundTripConstructor
from ruamel.yaml.representer import RoundTripRepresenter

from .template import JinjaEnvironment

if typing.TYPE_CHECKING:
    from ruamel.yaml.nodes import Node, ScalarNode
    from ruamel.yaml.constructor import Constructor
    from ruamel.yaml.representer import Representer

    from .app import Konverter


class UnsharedRoundTripConstructor(RoundTripConstructor):
    def __new__(cls, *args, **kwargs):
        klass = type(
            cls.__name__,
            (cls,),
            {
                "yaml_constructors": copy.deepcopy(cls.yaml_constructors),
                "yaml_multi_constructors": copy.deepcopy(cls.yaml_multi_constructors),
            },
        )
        return super().__new__(klass)


class UnsharedRoundTripRepresenter(RoundTripRepresenter):
    def __new__(cls, *args, **kwargs):
        klass = type(
            cls.__name__,
            (cls,),
            {
                "yaml_representers": copy.deepcopy(cls.yaml_representers),
                "yaml_multi_representers": copy.deepcopy(cls.yaml_multi_representers),
            },
        )
        return super().__new__(klass)


class BaseYAML(YAML):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.Representer = UnsharedRoundTripRepresenter
        self.Constructor = UnsharedRoundTripConstructor

    def register_type(self, klass: typing.Type[KonvertType]) -> None:
        self.representer.add_representer(
            klass, functools.partial(klass.to_yaml, yaml=self)
        )
        self.constructor.add_constructor(
            klass.yaml_tag, functools.partial(klass.from_yaml, yaml=self)
        )


class KonvertType(abc.ABC):
    yaml_tag: str

    def __init__(self, node: ScalarNode):
        self.node = node

    @classmethod
    def from_yaml(cls, constructor: Constructor, node: ScalarNode, yaml) -> typing.Any:
        return cls(node)

    @classmethod
    @abc.abstractmethod
    def to_yaml(cls, representer: Representer, instance: KonvertType, yaml) -> Node:
        pass


class KonvertExpression(KonvertType):
    yaml_tag = "!k/expr"

    @classmethod
    def to_yaml(cls, representer: Representer, instance: KonvertType, yaml) -> Node:
        return representer.represent_data(
            yaml.env.compile_expression(instance.node.value, undefined_to_none=False)(
                yaml.app.context
            )
        )


class KonvertTemplate(KonvertType):
    yaml_tag = "!k/template"

    @classmethod
    def to_yaml(cls, representer: Representer, instance: KonvertType, yaml) -> Node:
        value = yaml.env.eval_template(instance.node.value, yaml.app.context)
        return representer.represent_scalar(
            tag="tag:yaml.org,2002:str", value=value, style=instance.node.style
        )


class KonverterYAML(BaseYAML):
    def __init__(self, app: Konverter):
        super().__init__()
        self.app = app
        self.env = JinjaEnvironment(undefined=jinja2.StrictUndefined)
        self.register_type(KonvertExpression)
        self.register_type(KonvertTemplate)

    def render(self, in_file: typing.IO[str], out_file: typing.IO[str]) -> None:
        self.dump_all(self.load_all(in_file), out_file)



import click

from .app import Konverter


@click.command()
@click.argument("config_path", type=click.Path(exists=True, dir_okay=False))
def cli(config_path: str) -> None:
    app = Konverter.from_file(config_path)
    app.render(click.get_text_stream("stdout"))


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['konverter']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'cryptography>=2.8,<3.0',
 'jinja2>=2.10.3,<3.0.0',
 'ruamel.yaml>=0.16.5,<0.17.0']

entry_points = \
{'console_scripts': ['konverter = konverter.__main__:cli',
                     'konverter-vault = konverter.vault:cli']}

setup_kwargs = {
    'name': 'konverter',
    'version': '0.3.0',
    'description': 'YAML templating for responsible users',
    'long_description': '# Konverter - YAML templating for responsible users\n\nKonverter is a YAML templating tool that uses Jinja [expression](https://jinja.palletsprojects.com/en/2.10.x/templates/#expressions) and [templates](https://jinja.palletsprojects.com/en/2.10.x/templates/) and supports inline-encrypted variables.\n\n**Features:**\n- generic YAML templating (**not** specific to e.g. Kubernetes)\n- works on the **YAML structure**; text templating only available for values\n- support for simple **inline-encrypted variables**\n- ... more to come (see "[planned features](#planned-features)")\n\nThe project is still at an **early stage**, so I\'m happy about any contribution!\n\n### Motivation\n\nI created this tool mainly to scratch my own itch. I needed a simple way to store encrypted values along my Kubernetes manifests, but found that e.g. [Kustomize](https://kustomize.io/) makes this [extra hard](https://github.com/kubernetes-sigs/kustomize/issues/692).\n\nIn that regard Konverter follows the Python philosophy: "We are all [consenting adults](https://docs.python-guide.org/writing/style/#we-are-all-responsible-users)". So yes, you can do potentially dangerous things with it.\n\nAlso, the way Kustomize works by [patching and transforming](https://github.com/kubernetes-sigs/kustomize/blob/master/docs/fields.md#transformers) YAML, as well as the concept of `Secret` and `ConfigMap` [generators](https://github.com/kubernetes-sigs/kustomize/blob/master/docs/fields.md#generators), didn\'t feel natural to me.\n\nTo each his own :)\n\n## Installation\n\nThe easiest way to install Konverter is via [pipx](https://github.com/pipxproject/pipx):\n\n```shell\n$ pipx install konverter\n```\n\nOr just with plain pip:\n\n```shell\n$ pip install konverter\n```\n\n## Usage\n\nKonverter is configured via a YAML file (of course). In this config file, you define your templates and the context (variables) used to render those templates. If you want to render your templates with a different context (e.g. templating Kubernetes manifests for staging, production, ...) you need a separate config file.\n\nHere\'s a simple example of a config file:\n```yaml\ntemplates:\n  - manifest.yaml\n\ncontext:\n  - vars/common.yaml\n  - vars/prod.yaml\n```\n\nThe templates are rendered using the `konverter` command by passing it the config file:\n```shell\n$ konverter production.yaml\n...\n```\n\nThe rendered templates will be written to `stdout` as a multi-document YAML. This allows for easy composition with other tools like `kubectl`:\n\n```shell\n$ konverter production.yaml | kubectl apply -f -\n```\n\n### Templates\n\nUnder the `templates` key, you can configure a list of files or directories. In the case of a directory, Konverter will recursively include all `*.y[a]ml` files.\n\nKonverter uses [YAML tags](https://yaml.org/spec/1.2/spec.html#id2761292) to declare expressions (`!k/expr`) and templates (`!k/template`):\n\n```yaml\nfoo: !k/expr foo.bar\nFOO: !k/expr foo.bar|upper\n\nnot_defined: !k/expr missing_variable|default(\'default value\')\n\nconfig: !k/template |-\n  Hello {{ name }}\n\n  {% for item in list_of_values %}\n    Hello {{ item }}\n  {% endfor %}\n```\n\n#### Expressions\n\nExpressions are mainly useful for injecting data from the context and performing simple transformations:\n\n```yaml\n# Example context\nvalue:\n  from:\n    context: barfoo\n    other: foobar\n```\n\n```yaml\n# Use !k/expr to inject the values in a template\nfoo:\n  bar: !k/expr value.from.context\n  foo: !k/expr value.from.other|upper\n```\n\nWhen using the `!k/expr` tag, the value can be any valid [Jinja2 expression](https://jinja.palletsprojects.com/en/2.10.x/templates/#expressions). In addition to the standard Jinja [filters](https://jinja.palletsprojects.com/en/2.10.x/templates/#list-of-builtin-filters), [tests](https://jinja.palletsprojects.com/en/2.10.x/templates/#list-of-builtin-tests) and [global functions](https://jinja.palletsprojects.com/en/2.10.x/templates/#list-of-global-functions) the following filters are currently available:\n\n- `to_json`: dump an object as a *JSON string*\n- `b64encode`: encode value using Base64\n\nMissing a filter? Let me know or feel free to contribute!\n\n#### Templates\n\nThe `!k/template` tag supports the full [Jinja2 template syntax](https://jinja.palletsprojects.com/en/2.10.x/templates/) (currently with some exceptions, like template inheritance), but the output will always be a string value. Inline templates are most useful for customizing config files.\n\n### Context\n\nThe `context` key in the config contains a list of files or directories that Konverter uses for loading variable definitions. Those variables serve as the context (hence the name) when rendering templates. In the case of multiple context files containing the same top-level key, the entry from the last listed file wins (in the future we might support merging those contexts).\n\nThis is how a simple context file might look like\n```yaml\ndeployment:\n  name: foobar\n  server_name: foobar.example.com\nannotations:\n  version: 1.0\n  environment: production\n```\n\n#### Encrypted values\n\nIn addition there can be inline-encrypted values:\n\n```yaml\ncredentials:\n  user: root\n  password: !k/vault gAAAAABeKd7EEt-jCYJSLS_ze6oO401yRDCthJFkuR4ptIFNnSElTccUnOVSQ1rSCDbIdljB59SRWjy2rDq7174stq3FFzyE_w==\n```\n\nKonverter uses [Fernet symmetric encryption](https://cryptography.io/en/latest/fernet/) with secret keys. To use encrypted values we have to create key first:\n\n```shell\n$ konverter-vault keygen\n```\n\nThis will create a key file `.konverter-vault` in the current directory containing the key. **Make sure to never commit this file to version control!**\n\nBy default, Konverter will look for the vault key in the same directory as the config file when rendering templates. See the section "[advanced configuration](#advanced-configuration)" on how to use a different path.\n\nThe following command will decrypt any encrypted values in the given file and launch your `$EDITOR`:\n\n```shell\n$ konverter-vault edit vars/secret.yaml\n```\n\nValues that should be encrypted are marked with the `!k/encrypt` YAML tag:\n\n```yaml\ncredentials:\n  user: root\n  password: !k/encrypt secret-password\n```\n\nAfter closing the editor Konverter will encrypt the values:\n\n```yaml\ncredentials:\n  user: root\n  password: !k/vault gAAAAABeKd7EEt-jCYJSLS_ze6oO401yRDCthJFkuR4ptIFNnSElTccUnOVSQ1rSCDbIdljB59SRWjy2rDq7174stq3FFzyE_w==\n```\n\nInstead of editing the file via the `konverter-vault edit` command you can also encrypt/decrypt the files separately:\n\n```shell\n$ konverter-vault encrypt vars/secret.yaml\n...\n$ konverter-vault decrypt vars/secret.yaml\n```\n\nIn case the decrypted content should be passed to another tool that expects\nYAML or JSON, the `konverter-vault show` command can come in handy:\n\n```shell\n$ konverter-vault show --format=json vars/secret.yaml\n{\n  "credentials": {\n    "user": "root",\n    "password": "secret-password"\n  }\n}\n```\n\nIt will decrypt the given file and remove all `!k/(encrypt|vault)` YAML tags.\nSupported output formats are `yaml` (default), `json` or `terraform`.\n\nThe `terraform` output format is usefull for using `konverter-vault` as an\n[external data source in\nTerraform](https://www.terraform.io/docs/providers/external/data_source.html)\n\n```hcl\ndata "external" "konverter" {\n  program = [\n    "konverter-vault", "show", "--format=terraform", "vars/secrets.yaml"\n  ]\n}\n```\n\nUnfortunately the "[external program\nprotocol](https://www.terraform.io/docs/providers/external/data_source.html#external-program-protocol)"\nonly allows string values (no lists or objects). All `float`, `int` and `bool`\nvalues will be converted to strings. Other types will cause an error when\nusing this output format.\n\n### Advanced configuration\n\nThe above config file could be rewritten in a more verbose format:\n\n```yaml\ntemplates:\n  - manifest.yaml\n\nproviders:\n  default:\n    key_path: .konverter-vault\n\ncontext:\n  - provider: default\n    path: vars/common.yaml\n  - provider: default\n    path: vars/prod.yaml\n```\n\nIn case we want to change the location of the vault key, we can simply override the default provider:\n\n```yaml\n...\nproviders:\n  default:\n    key_path: ~/.my-vault-key\n...\n```\n\n## Planned features\n\n- better error handling\n- improve test coverage\n- importing templates from a file\n- support for other context providers\n\nFeel free to create an issue or a pull-request if you are missing something!\n',
    'author': 'Simon Westphahl',
    'author_email': 'simon@westphahl.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/westphahl/konverter',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
